@@ -397,6 +397,9 @@ void Surfaces::calculateEDT(Type type, float defaultResolution)
   if (!m_cube)
     m_cube = m_molecule->addCube();
 
+  // Set this cube as the active cube for volume rendering
+  m_molecule->setActiveCubeIndex(m_molecule->cubeCount() - 1);
+
   QFuture future = QtConcurrent::run([=]() {
     double probeRadius = 0.0;
     switch (type) {
@@ -589,6 +592,9 @@ void Surfaces::calculateQM(Type type, int index, bool beta, float isoValue,
   if (!m_cube)
     m_cube = m_molecule->addCube();
 
+  // Set this cube as the active cube for volume rendering
+  m_molecule->setActiveCubeIndex(m_molecule->cubeCount() - 1);
+
   if (type == Unknown)
     type = m_dialog->surfaceType();
 
@@ -687,6 +693,9 @@ void Surfaces::calculateCube(int index, float isoValue)
   if (m_cube == nullptr)
     return;
 
+  // Set this cube as the active cube for volume rendering
+  m_molecule->setActiveCubeIndex(index);
+
   if (isoValue == 0.0 && m_dialog != nullptr)
     m_isoValue = m_dialog->isosurfaceValue();
   else
@@ -861,8 +870,8 @@ void Surfaces::meshFinished()
 
     // finished, so request to enable the mesh display type
     QStringList displayTypes;
-    displayTypes << tr("Meshes");
-    requestActiveDisplayTypes(displayTypes);
+    displayTypes << tr("Surfaces");
+    emit requestActiveDisplayTypes(displayTypes);
 
     if (m_recordingMovie) {
       // Move to the next frame.
@@ -874,6 +883,9 @@ void Surfaces::meshFinished()
         m_dialog->reenableCalculateButton();
 
       m_molecule->emitChanged(QtGui::Molecule::Added);
+
+      emit commandFinished(tr(
+        "Surface Finished", "finished rendering surface or molecular orbital"));
     }
   }
 }
@@ -935,8 +947,9 @@ void Surfaces::movieFrame()
 
   auto glWidget = QtOpenGL::ActiveObjects::instance().activeGLWidget();
   if (!glWidget) {
-    QMessageBox::warning(qobject_cast<QWidget*>(parent()), tr("Avogadro"),
-                         "Couldn't find the active render widget, failing.");
+    QMessageBox::warning(
+      qobject_cast<QWidget*>(parent()), tr("Avogadro"),
+      tr("Couldn't find the active render widget, failing."));
     m_recordingMovie = false;
     m_dialog->enableRecord();
     return;
